#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <string>
#include <stdint.h>

using namespace std;

typedef vector<uintmax_t> Row;
typedef vector<Row> Matrix;

struct Transformation {
	virtual ~Transformation() {}
	virtual void apply(Matrix& matrix, uintmax_t w, uintmax_t h) {};
};

struct RowTransform : Transformation {
	size_t x, y;
	RowTransform(size_t x, size_t y) : x(x), y(y) {
	}
	virtual ~RowTransform() override {}
	void apply(Matrix& matrix, uintmax_t w, uintmax_t h) override {
		swap(matrix[x], matrix[y]);
	}
};

struct ColTransform : Transformation {
	size_t x, y;
	ColTransform(size_t x, size_t y) : x(x), y(y) {
	}
	virtual ~ColTransform() override {}
	void apply(Matrix& matrix, uintmax_t w, uintmax_t h) override {
		for (size_t i = 0; i < h; i++) {
			swap(matrix[i][x], matrix[i][y]);
		}
	}
};

struct CellTransform : Transformation {
	size_t a, b, c, d;
	CellTransform(size_t a, size_t b, size_t c, size_t d) : a(a), b(b), c(c), d(d) {
	}
	virtual ~CellTransform() override {}
	void apply(Matrix& matrix, uintmax_t w, uintmax_t h) override {
		swap(matrix[a][b], matrix[c][d]);
	}
};

Row expand(const Row& a, uintmax_t n, uintmax_t r) {
	Row res(n);
	auto k = a.size();
	std::copy(a.begin(), a.end(), res.begin());
	for (size_t i = k; i < n; i++) {
		res[i] = (10007 * res[i - 2] + 10009 * res[i - 1] + 87277) % r;
	}
	return std::move(res);
}

int main() {
	uintmax_t h, w, n, tmp;
	cin >> h >> w >> n;
	Matrix matrix(h, Row(w));
	for (uintmax_t i = 0; i < h; i++) {
		auto row = i*w;
		for (uintmax_t j = 0; j < w; j++) {
			matrix[i][j] = row + j;
		}
	}
	string s;
	cin >> s;
	uintmax_t lA, lB, lC, lD;
	cin >> lA;
	vector<uintmax_t> A(lA);
	for (uintmax_t i = 0; i < lA; i++)
		cin >> A[i];
	cin >> lB;
	vector<uintmax_t> B(lB);
	for (uintmax_t i = 0; i < lB; i++)
		cin >> B[i];
	cin >> lC;
	vector<uintmax_t> C(lC);
	for (uintmax_t i = 0; i < lC; i++)
		cin >> C[i];
	cin >> lD;
	vector<uintmax_t> D(lD);
	for (uintmax_t i = 0; i < lD; i++)
		cin >> D[i];
	Row Ax = expand(A, n, h);
	Row Bx = expand(B, n, w);
	Row Cx = expand(C, n, h);
	Row Dx = expand(D, n, w);
	auto slen = s.length();
	for (size_t i = 0; i < slen; i++) {
		auto c = s[i];
		switch (c) {
		case 'c':
			ColTransform(Bx[i], Dx[i]).apply(matrix, w, h);
			break;
		case 'r':
			RowTransform(Ax[i], Cx[i]).apply(matrix, w, h);
			break;
		case 'f':
			CellTransform(Ax[i], Bx[i], Cx[i], Dx[i]).apply(matrix, w, h);
			break;
		}
	}
	uintmax_t hash = 0;
	for (uintmax_t i = 0; i < h; i++) {
		auto row = uintmax_t(pow(17, i));
		for (uintmax_t j = 0; j < w; j++) {
			auto lol = matrix[i][j] * row*uintmax_t(pow(19, j));
			lol %= 1000000007ull;
			hash += lol;
		}
	}
	cout << hash << endl;
	return 0;
}